@@ -1,4 +1,4 @@
-# $Id: PMilter.pm,v 1.9.2.2 2004/02/24 18:54:33 tvierling Exp $
+# $Id: PMilter.pm,v 1.14.2.2 2004/03/06 15:31:12 tvierling Exp $
 #
 # Copyright (c) 2002-2004 Todd Vierling <tv@pobox.com> <tv@duh.org>
 # All rights reserved.
@@ -45,6 +45,7 @@ use Socket;
 use Symbol;
 use UNIVERSAL;
 
+our $VERSION = '0.92_01';
 our $DEBUG = 0;
 
 =pod
@@ -76,12 +77,6 @@ significantly, and the enhanced APIs and rule logic provided by PMilter
 0.5 and earlier has been factored out for inclusion in a separate package
 to be called Mail::Milter.
 
-So, in short, this package should be regarded as a drop-in replacement
-for Sendmail::Milter, with only some very minor API extensions related
-to the milter protocol itself.  (Sendmail::PMilter includes a wrapper
-module named Sendmail::Milter to allow old milter scripts to continue
-working as-is with this package.)
-
                NOTE NOTE NOTE NOTE NOTE NOTE NOTE
 
        This version of Sendmail::PMilter is NOT well tested.
@@ -121,10 +116,6 @@ our @EXPORT_OK = (@smflags, qw(
 ));
 our %EXPORT_TAGS = ( all => [ @smflags ] );
 
-##### Version of Sendmail::PMilter in use
-
-our $VERSION = '0.91_02';
-
 ##### Protocol constants
 
 # SMFIS_ are not the same as the standard, in order to keep "0" and "1"
@@ -687,7 +678,7 @@ sub ithread_dispatcher {
 
 	my $nchildren = 0;
 
-	threads::shared::share(\$nchildren);
+	threads::shared::share($nchildren);
 
 	sub {
 		my $this = shift;
@@ -695,7 +686,8 @@ sub ithread_dispatcher {
 		my $handler = shift;
 		my $maxchildren = $this->get_max_interpreters();
 
-		local $SIG{INFO} = sub {
+		my $siginfo = exists($SIG{INFO}) ? 'INFO' : 'USR1';
+		local $SIG{$siginfo} = sub {
 			print STDERR "Number of active children: $nchildren\n" if ($DEBUG > 0);
 		};
 
@@ -777,7 +769,9 @@ sub prefork_dispatcher () {
 		my $i = 0;
 
 		local $SIG{PIPE} = 'IGNORE'; # so close_callback will be reached
-		local $SIG{INFO} = sub {
+
+		my $siginfo = exists($SIG{INFO}) ? 'INFO' : 'USR1';
+		local $SIG{$siginfo} = sub {
 			print STDERR "$$: requests handled: $i\n" if ($DEBUG > 0);
 		};
 
@@ -796,13 +790,12 @@ sub prefork_dispatcher () {
 	my $killall = sub {
 		my $sig = shift;
 
-		kill -2, $$; # SIGINT to pgrp
-		$SIG{$sig} = 'DEFAULT';
+		kill 'TERM', keys %children;
+		exit 0;
 	};
-	local $SIG{HUP} = $killall;
 	local $SIG{INT} = $killall;
-	local $SIG{TERM} = $killall;
 	local $SIG{QUIT} = $killall;
+	local $SIG{TERM} = $killall;
 
 	setpgrp();
 
@@ -816,8 +809,16 @@ sub prefork_dispatcher () {
 				die "fork: $!" unless defined($pid);
 
 				if ($pid) {
+					# Perl reset these to IGNORE.  Restore them.
+					$SIG{INT} = $killall;
+					$SIG{QUIT} = $killall;
+					$SIG{TERM} = $killall;
 					$children{$pid} = 1;
 				} else {
+					# Perl reset these to IGNORE.  Set to defaults.
+					$SIG{INT} = 'DEFAULT';
+					$SIG{QUIT} = 'DEFAULT';
+					$SIG{TERM} = 'DEFAULT';
 					&$child_dispatcher(@_);
 					exit 0;
 				}
@@ -864,7 +865,9 @@ sub postfork_dispatcher () {
 
 		# Decrement child count on child exit.
 		local $SIG{CHLD} = $sigchld;
-		local $SIG{INFO} = sub {
+
+		my $siginfo = exists($SIG{INFO}) ? 'INFO' : 'USR1';
+		local $SIG{$siginfo} = sub {
 			print STDERR "Number of active children: $nchildren\n" if ($DEBUG > 0);
 		};
 
@@ -900,7 +903,7 @@ sub postfork_dispatcher () {
 				undef $lsocket;
 				undef $@;
 				$SIG{PIPE} = 'IGNORE'; # so close_callback will be reached
-				$SIG{INFO} = 'DEFAULT';
+				$SIG{$siginfo} = 'DEFAULT';
 
 				&$handler($socket);
 				$socket->close() if defined($socket);
